'============================================================================
'  Format PowerBASIC source code
'
'Author Bruce Brown, August 2010
'Small Changes: Theo Gottwald
' v2  changes : Paul Elliott May 2011
' v2a changed : Paul Elliott June 2011
'                got Global/Register/Dim/Local/Static in 1 routine
' v2b Peter
' v2c Paul       some changes to format Class code a little better
'                split some CASE statements out for easier maintenance
' v3d Peter      new IDE
'   e Paul       fix typos & input/output file handling
'     11/18/2011 rework adding comment beyond remcolumn
'                make UDT for variables to enhance formatting
'                no longer pass either UDT as they are global
'
'PBWIN 9
'Placed in public domain by author 8/26/10, all the usual disclaimers apply
'============================================================================
'Formats a Powerbasic source code file (BAS, INC) for consistent spacing and
'indentation. REM keywords are changed to a apostrophe (') and ASM keywords
'to !. Options provided to:
'     start all inline comments on a specific column
'     line-up the = sign of all declared equates on a specified column.
'     backup the source file prior to formatting
'     define the number of space for a tab
'     capitalize defined equates
'     split Global/Local variables to 1 per line
'Uses no global or static variables so easily could be converted to a DLL
'============================================================================

' >>PHNX_BEGIN_OPTIONS
'=========================================================================================
#DIM ALL
#REGISTER NONE
#COMPILE EXE
#OPTION VERSION5
'=========================================================================================
' >>PHNX_END_OPTIONS

' %UNICODE = 1
#IF %DEF(%UNICODE)
    MACRO XSTRING = WSTRING
    MACRO XASCIZ  = WSTRINGZ
    MACRO XASCIIZ = WSTRINGZ
#ELSE
    MACRO XSTRING = STRING
    MACRO XASCIZ = ASCIZ
    MACRO XASCIIZ = ASCIIZ
#ENDIF

MACRO XHWND = DWORD

MACRO S_TRIMBA(P1) = TRIM$(P1, ANY CHR$(0 TO 32))

MACRO Construct(P1, P2)
CASE P1
   IF Is_there(w(), P2, WordCount) THEN
      IF (WordNo = 1) THEN spacer = spacer + fo.tabsize
   END IF
END MACRO

MACRO m_ExitFunction(nresult)
  FUNCTION = nresult
  EXIT FUNCTION
END MACRO


' >>PHNX_BEGIN_WINAPI
'=========================================================================================
#INCLUDE "WIN32API.INC"
#INCLUDE "COMMCTRL.INC"
#INCLUDE "UTILITY.INC"
'=========================================================================================
' >>PHNX_END_WINAPI



TYPE fmtoptions
    capequates       AS BYTE                ' TRUE/FALSE capitalize equates
    lineupequates    AS BYTE                ' TRUE/FALSE position declared equates
    backup           AS BYTE                ' TRUE/FALSE backup source before formatting
    backupTimeStamp  AS BYTE                ' TRUE/FALSE Create a time stamp
    fspecin          AS XASCIZ * %MAX_PATH  ' source file spec
    fspecout         AS XASCIZ * %MAX_PATH  ' temporary output file spec
    remcol           AS LONG                ' column position for inline remarks
    equatescolumn    AS LONG                ' column pos of = in equates definitions
    tabsize          AS LONG                ' size of tab in characters
    splitvariables   AS LONG                ' split vars to seperate lines
    ascolumn         AS LONG                ' column for AS  from multiple variables
END TYPE

TYPE fmtcommon
    hprogress        AS DWORD               ' handle, progress window
    indent           AS LONG                ' left indenting helper
    inClass          AS LONG                ' needed to keep track if within Class
    fhin             AS LONG                ' file handle, source
    fhout            AS LONG                ' file handle output
END TYPE

GLOBAL fo AS FMTOPTIONS ' needs to be GLOBAL cuz of usage
GLOBAL fc AS FMTCOMMON ' 11/18/2011
GLOBAL tabhandle()  AS DWORD
GLOBAL mainhwnd     AS DWORD

$TEXTBTN3      = "&Help"
$TEXTBTNFORMAT = "&Format"
$CANCEL        = "E&xit"


' >>PHNX_BEGIN_CONSTANTS
'=========================================================================================
'---------------------------- [ Icon Identifiers ] -----------------------------

%IDI_PROGRAM_GROUP_GLOBE                     = 100

'--------------------------- [ Control Identifiers ] ---------------------------

' Form1
%IDD_FORM1                                   = 100
%IDC_FORM1_STATUSBAR1                        = 101
%IDC_FORM1_TAB1                              = 102
%IDC_FORM1_PAGE_FILE                         = 103
%IDC_FORM1_PAGE_OPTION                       = 104
%IDC_FORM1_BTN_HELP                          = 107
%IDC_FORM1_BTN_FORMAT                        = 108
' Page_File
%IDC_STATIC                                  = -1
%IDC_FORM1_TEXT_INPUTFILE                    = 105
%IDC_FORM1_TEXT_OUTPUTFILE                   = 106
%IDC_FORM1_BTN_INPUTFILE                     = 109
%IDC_FORM1_BTN_OUTPUTFILE                    = 110
' Page_Option
%IDC_FORM1_CHECK_CREATE_BAK                  = 111
%IDC_FORM1_CHECK_CREATE_TIME_STAMP           = 112
%IDC_FORM1_CHECK_SPLIT_VARIABLES             = 113
%IDC_FORM1_TEXT_TABSIZE                      = 114
%IDC_FORM1_TEXT_REMCOLUMN                    = 115
%IDC_FORM1_CHECK_FORMAT_EQULATES             = 116
%IDC_FORM1_CHECK_UPCASE_EQUATES              = 117
%IDC_FORM1_TEXT_EQUATECOLUMN                 = 118
%IDC_FORM1_BTN_SAVE_OPTION                   = 119
%IDC_FORM1_BTN_DEFAULT_OPTION                = 120
'=========================================================================================
' >>PHNX_END_CONSTANTS

%MAX_LINELEN                      = 400          'maximum source code line length

' >>PHNX_BEGIN_GLOBALS
'=========================================================================================
GLOBAL   ghInstance              AS DWORD    ' handle of the application instance

GLOBAL   gdwADM_ALIGNCONTROLS    AS DWORD    ' identifier of registered message
' Purpose: sent to a form or panel to reposition controls with alignment
'          styles after the form or panel has been resized or scrolled.
' wParam:  N/A
' lParam:  the low-order word is the width, and the high-order word the height of the client area
' Return:  N/A
'=========================================================================================
' >>PHNX_END_GLOBALS

' >>PHNX_BEGIN_DECLARES
'=========================================================================================
' Code Formatter project
' ----------------------
DECLARE FUNCTION CodeFormatter_RegisterClasses() AS LONG
DECLARE FUNCTION CodeFormatter_InitApplication(BYVAL hInstance AS DWORD, BYVAL lParam AS LONG) AS LONG
DECLARE SUB CodeFormatter_UnloadInstance(BYVAL hInstance AS DWORD)

' Form1 window
' ------------
DECLARE FUNCTION Form1_InitWindow(BYVAL hWnd AS DWORD, BYVAL lParam AS LONG) AS LONG
DECLARE FUNCTION Form1_TranslateMessage(BYVAL hWnd AS DWORD, tmsg AS tagMsg) AS LONG
DECLARE FUNCTION Form1_OnClose(BYVAL hWnd AS DWORD) AS LONG
DECLARE FUNCTION Form1_OnCreate(BYVAL hWnd AS DWORD, BYVAL lptcs AS DWORD, lMsgResult AS LONG) AS LONG
DECLARE SUB Form1_OnDestroy(BYVAL hWnd AS DWORD, lMsgResult AS LONG)
DECLARE FUNCTION Form1_OnQueryEndSession(BYVAL hWnd AS DWORD, BYVAL lSource AS LONG, BYVAL fLogOff AS LONG, lMsgResult AS LONG) AS LONG
DECLARE FUNCTION Form1_OnSize(BYVAL hWnd AS DWORD, BYVAL lState AS LONG, BYVAL cxClient AS LONG, BYVAL cyClient AS LONG) AS LONG
DECLARE FUNCTION Form1_OnPaint(BYVAL hWnd AS DWORD) AS LONG
DECLARE FUNCTION Form1_OnCommand(BYVAL hWnd AS DWORD, BYVAL lCtrlId AS LONG, BYVAL hWndCtrl AS DWORD, BYVAL lNotifyCode AS LONG) AS LONG
DECLARE FUNCTION Form1_OnNotify(BYVAL hWnd AS DWORD, BYVAL lCtrlId AS LONG, BYVAL lptnmhdr AS DWORD, lMsgResult AS LONG) AS LONG
DECLARE FUNCTION Form1_OnKillFocus(BYVAL hWnd AS DWORD, BYVAL hWndNewFocus AS DWORD) AS LONG
DECLARE FUNCTION Form1_OnSetFocus(BYVAL hWnd AS DWORD, BYVAL hWndOldFocus AS DWORD) AS LONG
DECLARE FUNCTION Form1_Tab1_SelChange(BYVAL hWndParent AS DWORD, BYVAL ptnmhdr AS NMHDR PTR, lMsgResult AS LONG) AS LONG
DECLARE FUNCTION Form1_Tab1_SelChanging(BYVAL hWndParent AS DWORD, BYVAL ptnmhdr AS NMHDR PTR, lMsgResult AS LONG) AS LONG
DECLARE FUNCTION Form1_Btn_Help_Clicked(BYVAL hWndParent AS DWORD, BYVAL hWndCtrl AS DWORD) AS LONG
DECLARE FUNCTION Form1_Btn_Format_Clicked(BYVAL hWndParent AS DWORD, BYVAL hWndCtrl AS DWORD) AS LONG
DECLARE FUNCTION Form1_Cancel_Clicked(BYVAL hWndParent AS DWORD, BYVAL hWndCtrl AS DWORD) AS LONG
DECLARE FUNCTION Form1_Page_File_OnCreate(BYVAL hWnd AS DWORD, BYVAL lptcs AS DWORD, lMsgResult AS LONG) AS LONG
DECLARE SUB Form1_Page_File_OnDestroy(BYVAL hWnd AS DWORD, lMsgResult AS LONG)
DECLARE FUNCTION Form1_Page_File_OnSize(BYVAL hWnd AS DWORD, BYVAL lState AS LONG, BYVAL cxClient AS LONG, BYVAL cyClient AS LONG) AS LONG
DECLARE FUNCTION Form1_Page_File_OnPaint(BYVAL hWnd AS DWORD) AS LONG
DECLARE FUNCTION Form1_Page_File_OnCommand(BYVAL hWnd AS DWORD, BYVAL lCtrlId AS LONG, BYVAL hWndCtrl AS DWORD, BYVAL lNotifyCode AS LONG) AS LONG
DECLARE FUNCTION Form1_Page_File_OnKillFocus(BYVAL hWnd AS DWORD, BYVAL hWndNewFocus AS DWORD) AS LONG
DECLARE FUNCTION Form1_Page_File_OnSetFocus(BYVAL hWnd AS DWORD, BYVAL hWndOldFocus AS DWORD) AS LONG
DECLARE FUNCTION Form1_Btn_InputFile_Clicked(BYVAL hWndParent AS DWORD, BYVAL hWndCtrl AS DWORD) AS LONG
DECLARE FUNCTION Form1_Btn_OutputFile_Clicked(BYVAL hWndParent AS DWORD, BYVAL hWndCtrl AS DWORD) AS LONG
DECLARE FUNCTION Form1_Text_InputFile_Change(BYVAL hWndParent AS DWORD, BYVAL hWndCtrl AS DWORD) AS LONG
DECLARE FUNCTION Form1_Text_OutputFile_Change(BYVAL hWndParent AS DWORD, BYVAL hWndCtrl AS DWORD) AS LONG
DECLARE FUNCTION Form1_Page_Option_OnCreate(BYVAL hWnd AS DWORD, BYVAL lptcs AS DWORD, lMsgResult AS LONG) AS LONG
DECLARE SUB Form1_Page_Option_OnDestroy(BYVAL hWnd AS DWORD, lMsgResult AS LONG)
DECLARE FUNCTION Form1_Page_Option_OnSize(BYVAL hWnd AS DWORD, BYVAL lState AS LONG, BYVAL cxClient AS LONG, BYVAL cyClient AS LONG) AS LONG
DECLARE FUNCTION Form1_Page_Option_OnPaint(BYVAL hWnd AS DWORD) AS LONG
DECLARE FUNCTION Form1_Page_Option_OnCommand(BYVAL hWnd AS DWORD, BYVAL lCtrlId AS LONG, BYVAL hWndCtrl AS DWORD, BYVAL lNotifyCode AS LONG) AS LONG
DECLARE FUNCTION Form1_Page_Option_OnKillFocus(BYVAL hWnd AS DWORD, BYVAL hWndNewFocus AS DWORD) AS LONG
DECLARE FUNCTION Form1_Page_Option_OnSetFocus(BYVAL hWnd AS DWORD, BYVAL hWndOldFocus AS DWORD) AS LONG
DECLARE FUNCTION Form1_Check1_Clicked(BYVAL hWndParent AS DWORD, BYVAL hWndCtrl AS DWORD) AS LONG
DECLARE FUNCTION Form1_Check2_Clicked(BYVAL hWndParent AS DWORD, BYVAL hWndCtrl AS DWORD) AS LONG
DECLARE FUNCTION Form1_Check3_Clicked(BYVAL hWndParent AS DWORD, BYVAL hWndCtrl AS DWORD) AS LONG
DECLARE FUNCTION Form1_Text_TabSize_Change(BYVAL hWndParent AS DWORD, BYVAL hWndCtrl AS DWORD) AS LONG
DECLARE FUNCTION Form1_Text_RemColumn_Change(BYVAL hWndParent AS DWORD, BYVAL hWndCtrl AS DWORD) AS LONG
DECLARE FUNCTION Form1_Check4_Clicked(BYVAL hWndParent AS DWORD, BYVAL hWndCtrl AS DWORD) AS LONG
DECLARE FUNCTION Form1_Check5_Clicked(BYVAL hWndParent AS DWORD, BYVAL hWndCtrl AS DWORD) AS LONG
DECLARE FUNCTION Form1_Text_EquateColumn_Change(BYVAL hWndParent AS DWORD, BYVAL hWndCtrl AS DWORD) AS LONG
DECLARE FUNCTION Form1_Btn_Save_Option_Clicked(BYVAL hWndParent AS DWORD, BYVAL hWndCtrl AS DWORD) AS LONG
DECLARE FUNCTION Form1_Btn_Default_Option_Clicked(BYVAL hWndParent AS DWORD, BYVAL hWndCtrl AS DWORD) AS LONG
'=========================================================================================
' >>PHNX_END_DECLARES

' >>PHNX_BEGIN_INCLUDES
'=========================================================================================
#INCLUDE "Form1.inc"

#RESOURCE "Code Formatter.PBR"
'=========================================================================================
' >>PHNX_END_INCLUDES

' >>PHNX_BEGIN_LOAD (programmer accessible procedures)
'=========================================================================================
'-------------------------------------------------------------------------------
'
' PROCEDURE: CodeFormatter_InitApplication
' PURPOSE:   Registers window classes and loads dynamic link libraries.
' RETURN:    FALSE if initialization was successful, TRUE if it was not.
'
'-------------------------------------------------------------------------------

FUNCTION CodeFormatter_InitApplication _
  ( _
  BYVAL hInstance AS DWORD, _ ' handle of current instance
  BYVAL lParam    AS LONG _   ' address of command line
  ) AS LONG

  LOCAL lpszcmdline AS XASCIZ PTR
  LOCAL filenum AS LONG
  LOCAL ztext AS XASCIZ * %MAX_PATH
  DIM Built AS GLOBAL IPOWERTIME

  LET Built = CLASS "PowerTime"

  Built.Now()

  REDIM tabhandle(0 TO 1)

  filenum = FREEFILE
  OPEN "Code Formatter.cfg" FOR BINARY AS filenum
  GET filenum,,fo
  CLOSE filenum

  lpszcmdline =  lParam

  'check the command line for a filespec
   IF LEN(@lpszcmdline) THEN
      IF ISFILE(@lpszcmdline) THEN
         fo.fspecin = @lpszcmdline               'assumes valid source file
      ELSE
         MessageBox 0, "File not found" + $CR + @lpszcmdline, "", %MB_ICONINFORMATION
         FUNCTION = %TRUE
         EXIT FUNCTION
      END IF
   ELSE
     FUNCTION = %FALSE
     EXIT FUNCTION
   END IF

END FUNCTION

'-------------------------------------------------------------------------------
'
' PROCEDURE: CodeFormatter_UnloadInstance
' PURPOSE:   Unloads dynamic link libraries and frees memory associated with
'            this instance.
'
'-------------------------------------------------------------------------------

SUB CodeFormatter_UnloadInstance _
  ( _
  BYVAL hInstance AS DWORD _  ' handle of current instance
  )

END SUB
'=========================================================================================
' >>PHNX_END_LOAD (programmer accessible procedures)

' >>PHNX_BEGIN_FORM
'=========================================================================================
'-------------------------------------------------------------------------------
'
' PROCEDURE: WinMain
' PURPOSE:   Program entry point, calls initialization function, processes
'            message loop.
'
'-------------------------------------------------------------------------------

FUNCTION WINMAIN _
   ( _
   BYVAL hInstance      AS DWORD, _       ' handle of current instance
   BYVAL hPrevInstance  AS DWORD, _       ' handle of previous instance(not used in Win32)
   BYVAL pszCmdLine     AS ASCIIZ PTR, _  ' address of command line
   BYVAL nCmdShow       AS LONG _         ' show state of window
   ) AS LONG

   LOCAL tmsg              AS tagMsg                     ' message information
   LOCAL ticc              AS INIT_COMMON_CONTROLSEX     ' specifies common control classes to register
   LOCAL hWnd              AS DWORD                      ' handle of main window
   LOCAL hWndModeless      AS DWORD                      ' handle of the current active window

   ' Save the handle of the application instance
   ghInstance = hInstance

   ' Load the common controls library and
   ' specify the classes to register.
   ticc.dwSize = SIZEOF(ticc)
   ticc.dwICC  = %ICC_BAR_CLASSES OR %ICC_TAB_CLASSES
   InitCommonControlsEx ticc

   ' Register custom messages
   gdwADM_ALIGNCONTROLS = RegisterWindowMessage("ADM_ALIGNCONTROLS")

   ' Register window classes
   IF ISTRUE CodeFormatter_RegisterClasses() THEN
      FUNCTION = %FALSE
      EXIT FUNCTION
   END IF

   ' Perform extra application initialization
   IF ISTRUE CodeFormatter_InitApplication(hInstance, BYVAL pszCmdLine) THEN
      FUNCTION = %FALSE
      EXIT FUNCTION
   END IF

   ' Create the Form1 window
   hWnd = CreateWindowEx(%NULL, _                                                      ' extended styles
                         "Form1_Class", _                                              ' class name
                         "Code Formatter PB 10 v3e", _                                 ' caption
                         %WS_OVERLAPPED OR %WS_MINIMIZE OR %WS_VISIBLE OR _            ' window styles
                         %WS_CAPTION OR %WS_MINIMIZEBOX OR %WS_SYSMENU, _
                         223, 170, _                                                   ' left, top
                         520, 445, _                                                   ' width, height
                         %NULL, %NULL, _                                               ' handle of owner, menu handle
                         ghInstance, BYVAL %NULL)                                      ' handle of instance, creation parameters
   ' If window could not be created, return "failure"
   IF ISFALSE hWnd THEN
      FUNCTION = %FALSE
      EXIT FUNCTION
   END IF

   ' Adjust the size of the window so that it conforms
   ' to the specified size of the client area
   phnxAdjustWindowRect hWnd, 514, 420

   ' Initialize this instance of the window
   IF ISTRUE Form1_InitWindow(hWnd, BYVAL pszCmdLine) THEN
      SendMessage hWnd, %WM_CLOSE, 0, 0
      FUNCTION = %FALSE
      EXIT FUNCTION
   END IF

   ' Center the window relative to the screen
   phnxCenterWindow hWnd, %NULL
   ' Make the window visible; update its client area
   ShowWindow hWnd, nCmdShow
   UpdateWindow hWnd

   ' Main message loop of program.
   ' Acquire and dispatch messages until a WM_QUIT message is received.
   WHILE ISTRUE GetMessage(tmsg, BYVAL %NULL, 0, 0)
      IF ISFALSE Form1_TranslateMessage(hWnd, tmsg) THEN
         hWndModeless = phnxGetFormHandle(GetFocus())
         IF (ISFALSE hWndModeless) OR (ISFALSE IsDialogMessage(hWndModeless, tmsg)) THEN
            TranslateMessage tmsg
            DispatchMessage tmsg
         END IF
      END IF
   WEND

   ' Free memory and unload dlls associated with this instance
   CodeFormatter_UnloadInstance hInstance

   FUNCTION = tmsg.wParam

END FUNCTION

'-------------------------------------------------------------------------------
'
' PROCEDURE: CodeFormatter_RegisterClasses
' PURPOSE:   Registers the window classes that are used in the program.
' RETURNS:   FALSE if successful, TRUE if registration failed.
'
'-------------------------------------------------------------------------------

FUNCTION CodeFormatter_RegisterClasses () AS LONG

   LOCAL szClassName    AS ASCIIZ * %MAX_PATH      ' class name
   LOCAL twcx           AS WNDCLASSEX              ' class information

   ' Register the window classes that belong to the Form1 window
   ' -----------------------------------------------------------

   ' Register the Form1 window
   szClassName        = "Form1_Class"
   twcx.cbSize        = SIZEOF(twcx)                                          ' size of WNDCLASSEX structure
   twcx.style         = %CS_DBLCLKS                                           ' class styles
   twcx.lpfnWndProc   = CODEPTR(Form1_WndProc)                                ' address of window procedure used by class
   twcx.cbClsExtra    = 0                                                     ' extra class bytes
   twcx.cbWndExtra    = 0                                                     ' extra window bytes
   twcx.hInstance     = ghInstance                                            ' instance of the process that is registering the window
   twcx.hIcon         = LoadIcon(ghInstance, BYVAL %IDI_PROGRAM_GROUP_GLOBE)  ' handle of class icon
   twcx.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)                   ' handle of class cursor
   twcx.hbrBackground = %COLOR_BTNFACE + 1                                    ' brush used to fill background of window's client area
   twcx.lpszMenuName  = %NULL                                                 ' resource identifier of the class menu
   twcx.lpszClassName = VARPTR(szClassName)                                   ' class name
   twcx.hIconSm       = %NULL                                                 ' handle of small icon shown in caption/system Taskbar
   IF ISFALSE RegisterClassEx(twcx) THEN
      FUNCTION = %TRUE
      EXIT FUNCTION
   END IF

   ' Register the Page_File window
   szClassName        = "Form1_Page_File_Class"
   twcx.cbSize        = SIZEOF(twcx)                              ' size of WNDCLASSEX structure
   twcx.style         = %CS_DBLCLKS                               ' class styles
   twcx.lpfnWndProc   = CODEPTR(Form1_Page_File_WndProc)          ' address of window procedure used by class
   twcx.cbClsExtra    = 0                                         ' extra class bytes
   twcx.cbWndExtra    = 0                                         ' extra window bytes
   twcx.hInstance     = ghInstance                                ' instance of the process that is registering the window
   twcx.hIcon         = %NULL                                     ' handle of class icon
   twcx.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)       ' handle of class cursor
   twcx.hbrBackground = %COLOR_BTNFACE + 1                        ' brush used to fill background of window's client area
   twcx.lpszMenuName  = %NULL                                     ' resource identifier of the class menu
   twcx.lpszClassName = VARPTR(szClassName)                       ' class name
   twcx.hIconSm       = %NULL                                     ' handle of small icon shown in caption/system Taskbar
   IF ISFALSE RegisterClassEx(twcx) THEN
      FUNCTION = %TRUE
      EXIT FUNCTION
   END IF

   ' Register the Page_Option window
   szClassName        = "Form1_Page_Option_Class"
   twcx.cbSize        = SIZEOF(twcx)                              ' size of WNDCLASSEX structure
   twcx.style         = %CS_DBLCLKS                               ' class styles
   twcx.lpfnWndProc   = CODEPTR(Form1_Page_Option_WndProc)        ' address of window procedure used by class
   twcx.cbClsExtra    = 0                                         ' extra class bytes
   twcx.cbWndExtra    = 0                                         ' extra window bytes
   twcx.hInstance     = ghInstance                                ' instance of the process that is registering the window
   twcx.hIcon         = %NULL                                     ' handle of class icon
   twcx.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)       ' handle of class cursor
   twcx.hbrBackground = %COLOR_BTNFACE + 1                        ' brush used to fill background of window's client area
   twcx.lpszMenuName  = %NULL                                     ' resource identifier of the class menu
   twcx.lpszClassName = VARPTR(szClassName)                       ' class name
   twcx.hIconSm       = %NULL                                     ' handle of small icon shown in caption/system Taskbar
   IF ISFALSE RegisterClassEx(twcx) THEN
      FUNCTION = %TRUE
      EXIT FUNCTION
   END IF

   ' All registration successful
   FUNCTION = %FALSE

END FUNCTION

'-------------------------------------------------------------------------------
'
' PROCEDURE: Form1_WndProc
' PURPOSE:   Processes messages for the Form1 window.
'
'-------------------------------------------------------------------------------

FUNCTION Form1_WndProc _
   ( _
   BYVAL hWnd     AS DWORD, _ ' window handle
   BYVAL uMsg     AS DWORD, _ ' type of message
   BYVAL wParam   AS DWORD, _ ' first message parameter
   BYVAL lParam   AS LONG _   ' second message parameter
   ) EXPORT AS LONG

   LOCAL szItem         AS ASCIIZ * %MAX_PATH      ' working variable
   LOCAL ttci           AS TC_ITEM                 ' specifies or receives the attributes of a tab
   LOCAL ptnmhdr        AS NMHDR PTR               ' information about a notification message
   LOCAL plEdge         AS LONG PTR                ' address of array of right edges
   LOCAL hWndChild      AS DWORD                   ' handle of child window
   LOCAL hFont          AS DWORD                   ' handle of font used by form
   LOCAL lMsgResult     AS LONG                    ' value returned to message after message is processed

   SELECT CASE uMsg
      CASE %WM_COMMAND

         SELECT CASE LOWRD(wParam)
            CASE %IDC_FORM1_BTN_HELP
               IF HIWRD(wParam) = %BN_CLICKED THEN
                  ' If the notification was processed
                  IF ISTRUE Form1_Btn_Help_Clicked(hWnd, lParam) THEN
                     FUNCTION = %FALSE
                     EXIT FUNCTION
                  END IF
               END IF

            CASE %IDC_FORM1_BTN_FORMAT
               IF HIWRD(wParam) = %BN_CLICKED THEN
                  ' If the notification was processed
                  IF ISTRUE Form1_Btn_Format_Clicked(hWnd, lParam) THEN
                     FUNCTION = %FALSE
                     EXIT FUNCTION
                  END IF
               END IF

            CASE %IDCANCEL
               IF HIWRD(wParam) = %BN_CLICKED THEN
                  ' If the notification was processed
                  IF ISTRUE Form1_Cancel_Clicked(hWnd, lParam) THEN
                     FUNCTION = %FALSE
                     EXIT FUNCTION
                  END IF
               END IF

            CASE ELSE
               ' If the message was processed
               IF ISTRUE Form1_OnCommand(hWnd, LOWRD(wParam), lParam, HIWRD(wParam)) THEN
                  FUNCTION = %FALSE
                  EXIT FUNCTION
               END IF
         END SELECT

      CASE %WM_NOTIFY
         ptnmhdr = lParam

         SELECT CASE @ptnmhdr.idFrom
            CASE %IDC_FORM1_TAB1

               SELECT CASE @ptnmhdr.code
                  CASE %TCN_SELCHANGE
                     ' If the notification was processed
                     IF ISTRUE Form1_Tab1_SelChange(hWnd, BYVAL ptnmhdr, lMsgResult) THEN
                        FUNCTION = lMsgResult
                        EXIT FUNCTION
                     END IF

                     ' Show the selected page
                     SELECT CASE SendMessage(@ptnmhdr.hwndFrom, %TCM_GETCURSEL, 0, 0)
                        CASE 0
                           ShowWindow GetDlgItem(hWnd, %IDC_FORM1_PAGE_FILE), %SW_SHOW

                        CASE 1
                           ShowWindow GetDlgItem(hWnd, %IDC_FORM1_PAGE_OPTION), %SW_SHOW
                     END SELECT

                     SetFocus @ptnmhdr.hwndFrom

                  CASE %TCN_SELCHANGING
                     ' If the notification was processed
                     IF ISTRUE Form1_Tab1_SelChanging(hWnd, BYVAL ptnmhdr, lMsgResult) THEN
                        FUNCTION = lMsgResult
                        EXIT FUNCTION
                     END IF

                     ' Hide the current page
                     SELECT CASE SendMessage(@ptnmhdr.hwndFrom, %TCM_GETCURSEL, 0, 0)
                        CASE 0
                           ShowWindow GetDlgItem(hWnd, %IDC_FORM1_PAGE_FILE), %SW_HIDE

                        CASE 1
                           ShowWindow GetDlgItem(hWnd, %IDC_FORM1_PAGE_OPTION), %SW_HIDE
                     END SELECT

               END SELECT

         END SELECT

         ' If the message was processed
         IF ISTRUE Form1_OnNotify(hWnd, wParam, lParam, lMsgResult) THEN
            FUNCTION = lMsgResult
            EXIT FUNCTION
         END IF

      CASE %WM_SYSCOLORCHANGE
         ' Forward this message to common controls so that they will
         ' be properly updated when the user changes the color settings.
         SendMessage GetDlgItem(hWnd, %IDC_FORM1_TAB1), %WM_SYSCOLORCHANGE, wParam, lParam
         SendMessage GetDlgItem(hWnd, %IDC_FORM1_STATUSBAR1), %WM_SYSCOLORCHANGE, wParam, lParam

      CASE %WM_SETTINGCHANGE
         ' Adjust the size of the window so that it conforms
         ' to the specified size of the client area
         phnxAdjustWindowRect hWnd, 514, 420

      CASE %WM_THEMECHANGED
         ' Adjust the size of the window so that it conforms
         ' to the specified size of the client area
         phnxAdjustWindowRect hWnd, 514, 420

      CASE %WM_SETFOCUS
         ' If the message was processed
         IF ISTRUE Form1_OnSetFocus(hWnd, wParam) THEN
            FUNCTION = %FALSE
            EXIT FUNCTION
         END IF
         ' Set the keyboard focus to the first control that is
         ' visible, not disabled, and has the WS_TABSTOP style
         SetFocus GetNextDlgTabItem(hWnd, %NULL, %FALSE)

      CASE %WM_KILLFOCUS
         ' If the message was processed
         IF ISTRUE Form1_OnKillFocus(hWnd, wParam) THEN
            FUNCTION = %FALSE
            EXIT FUNCTION
         END IF

      CASE %WM_CLOSE
         ' If the message was processed
         IF ISTRUE Form1_OnClose(hWnd) THEN
            FUNCTION = %FALSE
            EXIT FUNCTION
         END IF

      CASE %WM_QUERYENDSESSION
         ' If the message was processed
         IF ISTRUE Form1_OnQueryEndSession(hWnd, wParam, lParam, lMsgResult) THEN
            FUNCTION = lMsgResult
            EXIT FUNCTION
         END IF

      CASE %WM_DESTROY
         ' Destroy pages associated with tab control(s)
         DestroyWindow GetDlgItem(hWnd, %IDC_FORM1_PAGE_FILE)
         DestroyWindow GetDlgItem(hWnd, %IDC_FORM1_PAGE_OPTION)
         Form1_OnDestroy hWnd, lMsgResult
         PostQuitMessage lMsgResult
         FUNCTION = %FALSE
         EXIT FUNCTION

      CASE %WM_SIZE
         IF wParam <> %SIZE_MINIMIZED THEN
            ' Update the size and position of aligned controls
            SendMessage hWnd, gdwADM_ALIGNCONTROLS, wParam, lParam
         END IF
         ' If the message was processed
         IF ISTRUE Form1_OnSize(hWnd, wParam, LOWRD(lParam), HIWRD(lParam)) THEN
            FUNCTION = %FALSE
            EXIT FUNCTION
         END IF
         FUNCTION = %FALSE
         EXIT FUNCTION

      CASE gdwADM_ALIGNCONTROLS
         ' Update the size and position of aligned controls
         hWndChild = GetDlgItem(hWnd, %IDC_FORM1_STATUSBAR1)
         SendMessage hWndChild, %WM_SIZE, wParam, lParam
         InvalidateRect hWndChild, BYVAL %NULL, %TRUE
         FUNCTION = %FALSE
         EXIT FUNCTION

      CASE %WM_PAINT
         ' If the message was processed
         IF ISTRUE Form1_OnPaint(hWnd) THEN
            FUNCTION = %FALSE
            EXIT FUNCTION
         END IF

      CASE %WM_CREATE
         ' Create font used by container
         hFont = GetStockObject(%DEFAULT_GUI_FONT)

         ' Create the Statusbar1 statusbar control
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "msctls_statusbar32", _                               ' class name
                                    "", _                                                 ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR _                         ' window styles
                                    %CCS_BOTTOM OR %SBARS_SIZEGRIP, _                     ' class styles
                                    0, 400, _                                             ' left, top
                                    514, 20, _                                            ' width, height
                                    hWnd, %IDC_FORM1_STATUSBAR1, _                        ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Allocate memory for the coordinate of the right edge of each part
         plEdge = HeapAlloc(GetProcessHeap(), %HEAP_ZERO_MEMORY, 2 * 4)
         IF ISTRUE plEdge THEN
            @plEdge[0] = 96
            @plEdge[1] = 194
            SendMessage hWndChild, %SB_SETPARTS, 2, BYVAL plEdge
            ' Free memory that was allocated for the edge info
            HeapFree GetProcessHeap(), 0, BYVAL plEdge
         END IF
         ' Update the size of the statusbar
         SendMessage hWndChild, %WM_SIZE, 0, 0

         ' Create the Page_File window
         hWndChild = CreateWindowEx(%WS_EX_CONTROLPARENT, _                               ' extended styles
                                    "Form1_Page_File_Class", _                            ' class name
                                    "", _                                                 ' caption
                                    %WS_CHILD OR %WS_VISIBLE, _                           ' window styles
                                    15, 40, _                                             ' left, top
                                    481, 277, _                                           ' width, height
                                    hWnd, %IDC_FORM1_PAGE_FILE, _                         ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters

         ' Create the Page_Option window
         hWndChild = CreateWindowEx(%WS_EX_CONTROLPARENT, _                               ' extended styles
                                    "Form1_Page_Option_Class", _                          ' class name
                                    "", _                                                 ' caption
                                    %WS_CHILD, _                                          ' window styles
                                    15, 40, _                                             ' left, top
                                    481, 277, _                                           ' width, height
                                    hWnd, %IDC_FORM1_PAGE_OPTION, _                       ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters

         ' Create the Tab1 tab control
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "SysTabControl32", _                                  ' class name
                                    "", _                                                 ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_GROUP OR _            ' window styles
                                    %WS_TABSTOP OR _
                                    %TCS_TABS OR %TCS_SINGLELINE OR _                     ' class styles
                                    %TCS_RAGGEDRIGHT, _
                                    11, 18, _                                             ' left, top
                                    489, 303, _                                           ' width, height
                                    hWnd, %IDC_FORM1_TAB1, _                              ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Insert tabs
         szItem       = "File"
         ttci.mask    = %TCIF_TEXT
         ttci.pszText = VARPTR(szItem)
         SendMessage hWndChild, %TCM_INSERTITEM, 0, BYVAL VARPTR(ttci)

         szItem       = "Option"
         ttci.mask    = %TCIF_TEXT
         ttci.pszText = VARPTR(szItem)
         SendMessage hWndChild, %TCM_INSERTITEM, 1, BYVAL VARPTR(ttci)

         ' Create the Btn_Help text button
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "Button", _                                           ' class name
                                    "&Hilfe", _                                           ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _          ' window styles
                                    %BS_PUSHBUTTON OR %BS_CENTER OR %BS_VCENTER, _        ' class styles
                                    15, 335, _                                            ' left, top
                                    116, 41, _                                            ' width, height
                                    hWnd, %IDC_FORM1_BTN_HELP, _                          ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the Btn_Format text button
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "Button", _                                           ' class name
                                    "&Format", _                                          ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _          ' window styles
                                    %BS_PUSHBUTTON OR %BS_CENTER OR %BS_VCENTER, _        ' class styles
                                    195, 335, _                                           ' left, top
                                    116, 41, _                                            ' width, height
                                    hWnd, %IDC_FORM1_BTN_FORMAT, _                        ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the Cancel text button
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "Button", _                                           ' class name
                                    "Abbrechen", _                                        ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _          ' window styles
                                    %BS_PUSHBUTTON OR %BS_CENTER OR %BS_VCENTER, _        ' class styles
                                    380, 335, _                                           ' left, top
                                    116, 41, _                                            ' width, height
                                    hWnd, %IDCANCEL, _                                    ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' If the message was processed
         IF ISTRUE Form1_OnCreate(hWnd, lParam, lMsgResult) THEN
            FUNCTION = lMsgResult
            EXIT FUNCTION
         END IF
         FUNCTION = %FALSE
         EXIT FUNCTION
   END SELECT

   FUNCTION = DefWindowProc(hWnd, uMsg, wParam, lParam)

END FUNCTION

'-------------------------------------------------------------------------------
'
' PROCEDURE: Form1_Page_File_WndProc
' PURPOSE:   Processes messages for the Page_File window.
'
'-------------------------------------------------------------------------------

FUNCTION Form1_Page_File_WndProc _
   ( _
   BYVAL hWnd     AS DWORD, _ ' window handle
   BYVAL uMsg     AS DWORD, _ ' type of message
   BYVAL wParam   AS DWORD, _ ' first message parameter
   BYVAL lParam   AS LONG _   ' second message parameter
   ) EXPORT AS LONG

   LOCAL hWndChild      AS DWORD    ' handle of child window
   LOCAL hFont          AS DWORD    ' handle of font used by panel
   LOCAL lMsgResult     AS LONG     ' value returned to message after message is processed

   SELECT CASE uMsg
      CASE %WM_COMMAND

         SELECT CASE LOWRD(wParam)
            CASE %IDC_FORM1_BTN_INPUTFILE
               IF HIWRD(wParam) = %BN_CLICKED THEN
                  ' If the notification was processed
                  IF ISTRUE Form1_Btn_InputFile_Clicked(hWnd, lParam) THEN
                     FUNCTION = %FALSE
                     EXIT FUNCTION
                  END IF
               END IF

            CASE %IDC_FORM1_BTN_OUTPUTFILE
               IF HIWRD(wParam) = %BN_CLICKED THEN
                  ' If the notification was processed
                  IF ISTRUE Form1_Btn_OutputFile_Clicked(hWnd, lParam) THEN
                     FUNCTION = %FALSE
                     EXIT FUNCTION
                  END IF
               END IF

            CASE %IDC_FORM1_TEXT_INPUTFILE
               IF HIWRD(wParam) = %EN_CHANGE THEN
                  ' If the notification was processed
                  IF ISTRUE Form1_Text_InputFile_Change(hWnd, lParam) THEN
                     FUNCTION = %FALSE
                     EXIT FUNCTION
                  END IF
               END IF

            CASE %IDC_FORM1_TEXT_OUTPUTFILE
               IF HIWRD(wParam) = %EN_CHANGE THEN
                  ' If the notification was processed
                  IF ISTRUE Form1_Text_OutputFile_Change(hWnd, lParam) THEN
                     FUNCTION = %FALSE
                     EXIT FUNCTION
                  END IF
               END IF

            CASE ELSE
               ' If the message was processed
               IF ISTRUE Form1_Page_File_OnCommand(hWnd, LOWRD(wParam), lParam, HIWRD(wParam)) THEN
                  FUNCTION = %FALSE
                  EXIT FUNCTION
               END IF
         END SELECT

      CASE %WM_SETFOCUS
         ' If the message was processed
         IF ISTRUE Form1_Page_File_OnSetFocus(hWnd, wParam) THEN
            FUNCTION = %FALSE
            EXIT FUNCTION
         END IF

      CASE %WM_KILLFOCUS
         ' If the message was processed
         IF ISTRUE Form1_Page_File_OnKillFocus(hWnd, wParam) THEN
            FUNCTION = %FALSE
            EXIT FUNCTION
         END IF

      CASE %WM_DESTROY
         Form1_Page_File_OnDestroy hWnd, lMsgResult
         FUNCTION = %FALSE
         EXIT FUNCTION

      CASE %WM_SIZE
         ' If the message was processed
         IF ISTRUE Form1_Page_File_OnSize(hWnd, wParam, LOWRD(lParam), HIWRD(lParam)) THEN
            FUNCTION = %FALSE
            EXIT FUNCTION
         END IF

      CASE %WM_PAINT
         ' If the message was processed
         IF ISTRUE Form1_Page_File_OnPaint(hWnd) THEN
            FUNCTION = %FALSE
            EXIT FUNCTION
         END IF

      CASE %WM_CREATE
         ' Create font used by container
         hFont = GetStockObject(%DEFAULT_GUI_FONT)

         ' Create the TextLbl1 label
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "Static", _                                           ' class name
                                    "Input File", _                                       ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_GROUP OR _            ' window styles
                                    %SS_LEFT, _                                           ' class styles
                                    15, 25, _                                             ' left, top
                                    66, 16, _                                             ' width, height
                                    hWnd, %IDC_STATIC, _                                  ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the TextLbl2 label
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "Static", _                                           ' class name
                                    "Output File", _                                      ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_GROUP OR _            ' window styles
                                    %SS_LEFT, _                                           ' class styles
                                    15, 60, _                                             ' left, top
                                    61, 16, _                                             ' width, height
                                    hWnd, %IDC_STATIC, _                                  ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the Btn_InputFile text button
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "Button", _                                           ' class name
                                    "...", _                                              ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _          ' window styles
                                    %BS_PUSHBUTTON OR %BS_CENTER OR %BS_VCENTER, _        ' class styles
                                    445, 25, _                                            ' left, top
                                    26, 21, _                                             ' width, height
                                    hWnd, %IDC_FORM1_BTN_INPUTFILE, _                     ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the Btn_OutputFile text button
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "Button", _                                           ' class name
                                    "...", _                                              ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _          ' window styles
                                    %BS_PUSHBUTTON OR %BS_CENTER OR %BS_VCENTER, _        ' class styles
                                    445, 60, _                                            ' left, top
                                    26, 21, _                                             ' width, height
                                    hWnd, %IDC_FORM1_BTN_OUTPUTFILE, _                    ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the Text_InputFile edit control
         hWndChild = CreateWindowEx(%WS_EX_CLIENTEDGE, _                                  ' extended styles
                                    "Edit", _                                             ' class name
                                    "", _                                                 ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _          ' window styles
                                    %ES_LEFT OR %ES_AUTOHSCROLL, _                        ' class styles
                                    95, 25, _                                             ' left, top
                                    341, 21, _                                            ' width, height
                                    hWnd, %IDC_FORM1_TEXT_INPUTFILE, _                    ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the Text_OutputFile edit control
         hWndChild = CreateWindowEx(%WS_EX_CLIENTEDGE, _                                  ' extended styles
                                    "Edit", _                                             ' class name
                                    "", _                                                 ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _          ' window styles
                                    %ES_LEFT OR %ES_AUTOHSCROLL, _                        ' class styles
                                    95, 60, _                                             ' left, top
                                    341, 21, _                                            ' width, height
                                    hWnd, %IDC_FORM1_TEXT_OUTPUTFILE, _                   ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' If the message was processed
         IF ISTRUE Form1_Page_File_OnCreate(hWnd, lParam, lMsgResult) THEN
            FUNCTION = lMsgResult
            EXIT FUNCTION
         END IF
         FUNCTION = %FALSE
         EXIT FUNCTION
   END SELECT

   FUNCTION = DefWindowProc(hWnd, uMsg, wParam, lParam)

END FUNCTION

'-------------------------------------------------------------------------------
'
' PROCEDURE: Form1_Page_Option_WndProc
' PURPOSE:   Processes messages for the Page_Option window.
'
'-------------------------------------------------------------------------------

FUNCTION Form1_Page_Option_WndProc _
   ( _
   BYVAL hWnd     AS DWORD, _ ' window handle
   BYVAL uMsg     AS DWORD, _ ' type of message
   BYVAL wParam   AS DWORD, _ ' first message parameter
   BYVAL lParam   AS LONG _   ' second message parameter
   ) EXPORT AS LONG

   LOCAL hWndChild      AS DWORD    ' handle of child window
   LOCAL hFont          AS DWORD    ' handle of font used by panel
   LOCAL lMsgResult     AS LONG     ' value returned to message after message is processed

   SELECT CASE uMsg
      CASE %WM_COMMAND

         SELECT CASE LOWRD(wParam)
            CASE %IDC_FORM1_CHECK_CREATE_BAK
               IF HIWRD(wParam) = %BN_CLICKED THEN
                  ' If the notification was processed
                  IF ISTRUE Form1_Check1_Clicked(hWnd, lParam) THEN
                     FUNCTION = %FALSE
                     EXIT FUNCTION
                  END IF
               END IF

            CASE %IDC_FORM1_CHECK_CREATE_TIME_STAMP
               IF HIWRD(wParam) = %BN_CLICKED THEN
                  ' If the notification was processed
                  IF ISTRUE Form1_Check2_Clicked(hWnd, lParam) THEN
                     FUNCTION = %FALSE
                     EXIT FUNCTION
                  END IF
               END IF

            CASE %IDC_FORM1_CHECK_SPLIT_VARIABLES
               IF HIWRD(wParam) = %BN_CLICKED THEN
                  ' If the notification was processed
                  IF ISTRUE Form1_Check3_Clicked(hWnd, lParam) THEN
                     FUNCTION = %FALSE
                     EXIT FUNCTION
                  END IF
               END IF

            CASE %IDC_FORM1_TEXT_TABSIZE
               IF HIWRD(wParam) = %EN_CHANGE THEN
                  ' If the notification was processed
                  IF ISTRUE Form1_Text_TabSize_Change(hWnd, lParam) THEN
                     FUNCTION = %FALSE
                     EXIT FUNCTION
                  END IF
               END IF

            CASE %IDC_FORM1_TEXT_REMCOLUMN
               IF HIWRD(wParam) = %EN_CHANGE THEN
                  ' If the notification was processed
                  IF ISTRUE Form1_Text_RemColumn_Change(hWnd, lParam) THEN
                     FUNCTION = %FALSE
                     EXIT FUNCTION
                  END IF
               END IF

            CASE %IDC_FORM1_CHECK_FORMAT_EQULATES
               IF HIWRD(wParam) = %BN_CLICKED THEN
                  ' If the notification was processed
                  IF ISTRUE Form1_Check4_Clicked(hWnd, lParam) THEN
                     FUNCTION = %FALSE
                     EXIT FUNCTION
                  END IF
               END IF

            CASE %IDC_FORM1_CHECK_UPCASE_EQUATES
               IF HIWRD(wParam) = %BN_CLICKED THEN
                  ' If the notification was processed
                  IF ISTRUE Form1_Check5_Clicked(hWnd, lParam) THEN
                     FUNCTION = %FALSE
                     EXIT FUNCTION
                  END IF
               END IF

            CASE %IDC_FORM1_TEXT_EQUATECOLUMN
               IF HIWRD(wParam) = %EN_CHANGE THEN
                  ' If the notification was processed
                  IF ISTRUE Form1_Text_EquateColumn_Change(hWnd, lParam) THEN
                     FUNCTION = %FALSE
                     EXIT FUNCTION
                  END IF
               END IF

            CASE %IDC_FORM1_BTN_SAVE_OPTION
               IF HIWRD(wParam) = %BN_CLICKED THEN
                  ' If the notification was processed
                  IF ISTRUE Form1_Btn_Save_Option_Clicked(hWnd, lParam) THEN
                     FUNCTION = %FALSE
                     EXIT FUNCTION
                  END IF
               END IF

            CASE %IDC_FORM1_BTN_DEFAULT_OPTION
               IF HIWRD(wParam) = %BN_CLICKED THEN
                  ' If the notification was processed
                  IF ISTRUE Form1_Btn_Default_Option_Clicked(hWnd, lParam) THEN
                     FUNCTION = %FALSE
                     EXIT FUNCTION
                  END IF
               END IF

            CASE ELSE
               ' If the message was processed
               IF ISTRUE Form1_Page_Option_OnCommand(hWnd, LOWRD(wParam), lParam, HIWRD(wParam)) THEN
                  FUNCTION = %FALSE
                  EXIT FUNCTION
               END IF
         END SELECT

      CASE %WM_SETFOCUS
         ' If the message was processed
         IF ISTRUE Form1_Page_Option_OnSetFocus(hWnd, wParam) THEN
            FUNCTION = %FALSE
            EXIT FUNCTION
         END IF

      CASE %WM_KILLFOCUS
         ' If the message was processed
         IF ISTRUE Form1_Page_Option_OnKillFocus(hWnd, wParam) THEN
            FUNCTION = %FALSE
            EXIT FUNCTION
         END IF

      CASE %WM_DESTROY
         Form1_Page_Option_OnDestroy hWnd, lMsgResult
         FUNCTION = %FALSE
         EXIT FUNCTION

      CASE %WM_SIZE
         ' If the message was processed
         IF ISTRUE Form1_Page_Option_OnSize(hWnd, wParam, LOWRD(lParam), HIWRD(lParam)) THEN
            FUNCTION = %FALSE
            EXIT FUNCTION
         END IF

      CASE %WM_PAINT
         ' If the message was processed
         IF ISTRUE Form1_Page_Option_OnPaint(hWnd) THEN
            FUNCTION = %FALSE
            EXIT FUNCTION
         END IF

      CASE %WM_CREATE
         ' Create font used by container
         hFont = GetStockObject(%DEFAULT_GUI_FONT)

         ' Create the Check1 checkbox
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "Button", _                                           ' class name
                                    "Create Bak", _                                       ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _          ' window styles
                                    %BS_AUTOCHECKBOX OR %BS_LEFT OR %BS_VCENTER OR _      ' class styles
                                    %BS_RIGHTBUTTON, _
                                    25, 25, _                                             ' left, top
                                    151, 21, _                                            ' width, height
                                    hWnd, %IDC_FORM1_CHECK_CREATE_BAK, _                  ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the Check2 checkbox
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "Button", _                                           ' class name
                                    "Create Time Stamp", _                                ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _          ' window styles
                                    %BS_AUTOCHECKBOX OR %BS_LEFT OR %BS_VCENTER OR _      ' class styles
                                    %BS_RIGHTBUTTON, _
                                    25, 65, _                                             ' left, top
                                    151, 21, _                                            ' width, height
                                    hWnd, %IDC_FORM1_CHECK_CREATE_TIME_STAMP, _           ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the Check3 checkbox
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "Button", _                                           ' class name
                                    "Split Variables extr line", _                        ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _          ' window styles
                                    %BS_AUTOCHECKBOX OR %BS_LEFT OR %BS_VCENTER OR _      ' class styles
                                    %BS_RIGHTBUTTON, _
                                    25, 105, _                                            ' left, top
                                    151, 21, _                                            ' width, height
                                    hWnd, %IDC_FORM1_CHECK_SPLIT_VARIABLES, _             ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the TextLbl3 label
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "Static", _                                           ' class name
                                    "Tab Size", _                                         ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_GROUP OR _            ' window styles
                                    %SS_LEFT, _                                           ' class styles
                                    25, 150, _                                            ' left, top
                                    56, 21, _                                             ' width, height
                                    hWnd, %IDC_STATIC, _                                  ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the Text_TabSize edit control
         hWndChild = CreateWindowEx(%WS_EX_CLIENTEDGE, _                                  ' extended styles
                                    "Edit", _                                             ' class name
                                    "3", _                                                ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _          ' window styles
                                    %ES_RIGHT OR %ES_AUTOHSCROLL OR %ES_NUMBER, _         ' class styles
                                    140, 145, _                                           ' left, top
                                    41, 21, _                                             ' width, height
                                    hWnd, %IDC_FORM1_TEXT_TABSIZE, _                      ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the TextLbl4 label
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "Static", _                                           ' class name
                                    "Rem Column", _                                       ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_GROUP OR _            ' window styles
                                    %SS_LEFT, _                                           ' class styles
                                    25, 185, _                                            ' left, top
                                    76, 21, _                                             ' width, height
                                    hWnd, %IDC_STATIC, _                                  ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the Text_RemColumn edit control
         hWndChild = CreateWindowEx(%WS_EX_CLIENTEDGE, _                                  ' extended styles
                                    "Edit", _                                             ' class name
                                    "50", _                                               ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _          ' window styles
                                    %ES_RIGHT OR %ES_AUTOHSCROLL OR %ES_NUMBER, _         ' class styles
                                    140, 185, _                                           ' left, top
                                    41, 21, _                                             ' width, height
                                    hWnd, %IDC_FORM1_TEXT_REMCOLUMN, _                    ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the Check4 checkbox
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "Button", _                                           ' class name
                                    "Format", _                                           ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _          ' window styles
                                    %BS_AUTOCHECKBOX OR %BS_LEFT OR %BS_VCENTER OR _      ' class styles
                                    %BS_RIGHTBUTTON, _
                                    250, 50, _                                            ' left, top
                                    151, 21, _                                            ' width, height
                                    hWnd, %IDC_FORM1_CHECK_FORMAT_EQULATES, _             ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the Check5 checkbox
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "Button", _                                           ' class name
                                    "UpCase", _                                           ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _          ' window styles
                                    %BS_AUTOCHECKBOX OR %BS_LEFT OR %BS_VCENTER OR _      ' class styles
                                    %BS_RIGHTBUTTON, _
                                    250, 85, _                                            ' left, top
                                    151, 21, _                                            ' width, height
                                    hWnd, %IDC_FORM1_CHECK_UPCASE_EQUATES, _              ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the TextLbl5 label
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "Static", _                                           ' class name
                                    "Column", _                                           ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_GROUP OR _            ' window styles
                                    %SS_LEFT, _                                           ' class styles
                                    250, 125, _                                           ' left, top
                                    61, 21, _                                             ' width, height
                                    hWnd, %IDC_STATIC, _                                  ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the Text_EquateColumn edit control
         hWndChild = CreateWindowEx(%WS_EX_CLIENTEDGE, _                                  ' extended styles
                                    "Edit", _                                             ' class name
                                    "35", _                                               ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _          ' window styles
                                    %ES_RIGHT OR %ES_AUTOHSCROLL OR %ES_NUMBER, _         ' class styles
                                    365, 125, _                                           ' left, top
                                    41, 21, _                                             ' width, height
                                    hWnd, %IDC_FORM1_TEXT_EQUATECOLUMN, _                 ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the Group1 groupbox
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "Button", _                                           ' class name
                                    "Equlates", _                                         ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_CLIPSIBLINGS OR _     ' window styles
                                    %WS_GROUP OR _
                                    %BS_GROUPBOX, _                                       ' class styles
                                    235, 25, _                                            ' left, top
                                    231, 136, _                                           ' width, height
                                    hWnd, %IDC_STATIC, _                                  ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the Btn_Save_Option text button
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "Button", _                                           ' class name
                                    "Save Option", _                                      ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _          ' window styles
                                    %BS_PUSHBUTTON OR %BS_CENTER OR %BS_VCENTER, _        ' class styles
                                    345, 190, _                                           ' left, top
                                    121, 21, _                                            ' width, height
                                    hWnd, %IDC_FORM1_BTN_SAVE_OPTION, _                   ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' Create the Btn_Default_Option text button
         hWndChild = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "Button", _                                           ' class name
                                    "Default Option", _                                   ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR _          ' window styles
                                    %BS_PUSHBUTTON OR %BS_CENTER OR %BS_VCENTER, _        ' class styles
                                    345, 220, _                                           ' left, top
                                    121, 21, _                                            ' width, height
                                    hWnd, %IDC_FORM1_BTN_DEFAULT_OPTION, _                ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hWndChild, %WM_SETFONT, hFont, %TRUE

         ' If the message was processed
         IF ISTRUE Form1_Page_Option_OnCreate(hWnd, lParam, lMsgResult) THEN
            FUNCTION = lMsgResult
            EXIT FUNCTION
         END IF
         FUNCTION = %FALSE
         EXIT FUNCTION
   END SELECT

   FUNCTION = DefWindowProc(hWnd, uMsg, wParam, lParam)

END FUNCTION
'=========================================================================================
' >>PHNX_END_FORM



FUNCTION CaptionProgressWindow(BYVAL hparent AS DWORD, BYVAL PBCAPTION AS XSTRING) AS DWORD
   '--------------------------------------------------------------------------
   'Create a centered progress control with caption and return handle
   '--------------------------------------------------------------------------
   LOCAL rc AS RECT
   LOCAL hwin AS DWORD                           'progress control handle
   LOCAL ht, wd AS LONG                          'window height & width
   'fetch desktop size
   SystemParametersInfo %SPI_GETWORKAREA, 0, rc, 0
   ht = GetSystemMetrics(%SM_CYCAPTION)          'height of window caption
   wd = rc.nright \ 2                            '1/2 of screen width
   IF hparent = 0 THEN
      hparent = GetDesktopWindow
   END IF
   hwin = CreateWindowEx(%WS_EX_TOOLWINDOW OR %WS_EX_DLGMODALFRAME, "msctls_progress32", _
   BYVAL STRPTR(PBCAPTION), %WS_VISIBLE OR %WS_CAPTION OR %WS_POPUP OR %PBS_SMOOTH, _
   (rc.nright \ 2) - (wd \ 2), _                 'horiz pos start
   (rc.nbottom \ 2) - ht, _                      'vert  pos start
   wd, _                                         'width, 1/2 screen
   ht * 2, _                                     'height, 2 * caption
   hparent, 0, GetModuleHandle(BYVAL %NULL), BYVAL %NULL)
   FUNCTION = hwin                               'return progress handle
END FUNCTION

SUB DoFormat ' 11/18/2011 (BYVAL fo AS FMTOPTIONS PTR)
   '----------------------------------------------------------------------------
   'Format the source file. The w(1-x) array holds words, tokens, strings
   'of parsed source line data. w(0) element is reserved for any in-line comment
   'that doesn't fit to be placed above the affected line.
   '----------------------------------------------------------------------------
   LOCAL Wordcount AS LONG                       '# tokens in source line
   LOCAL CurrentLine AS LONG                     '0 based current edit line number
   LOCAL TotalLines AS LONG                      'total # lines in control
   LOCAL ztext AS ASCIZ * %MAX_LINELEN           'source code line
   DIM w(100) AS LOCAL STRING                    'words/tokens in line
   LOCAL build AS STRING                         'rebuilt source line

   ' 05/06/2011 PDE expand GLOBAL/LOCAL to 1 variable per line
   LOCAL str1 AS STRING
   LOCAL str2 AS STRING
   LOCAL str3 AS STRING
   LOCAL strBgn AS STRING
   LOCAL strAs AS STRING

   DIM   vars(25) AS STRING
   LOCAL varstr AS STRING
   LOCAL varfixsize AS STRING
   LOCAL varcnt AS LONG
   LOCAL varDeclare AS STRING
   LOCAL varDeclareSize AS LONG

   LOCAL a AS LONG
   LOCAL b AS LONG
   LOCAL x AS LONG
   LOCAL y AS LONG
   LOCAL z AS LONG
   LOCAL preAS AS LONG

   '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   'Start formatting, fetch each source line, parse into
   'tokens/words, rebuild it according to user preferences,
   'and add to temp file.
   '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   FILESCAN fc.fhin, RECORDS TO TotalLines      'total source code lines
   FOR currentline = 1 TO totallines
      LINE INPUT #fc.fhin, ztext

      IF INSTR(UCASE$(ztext), "#PBFORMS") THEN
         GOTO NoGlobalLocal1
      END IF

      IF fo.splitvariables = 0 THEN
         GOTO NoGlobalLocal
      END IF

      ' 06/21/2011 PDE split up Global/Local to 1 variable per line
      str1 = ztext

      str2 = TRIM$(str1)
      str2 = UCASE$(str2)

      preAS = fo.ascolumn - 2

      str2 = TRIM$(str1)
      str2 = UCASE$(str2)
      varDeclare = ""
      varDeclareSize = 0
      IF LEFT$(str2, 7) = "GLOBAL " THEN
         varDeclareSize = 7
      END IF
      IF LEFT$(str2, 9) = "REGISTER " THEN
         varDeclareSize = 9
      END IF
      IF LEFT$(str2, 4) = "DIM " THEN
         varDeclareSize = 4
      END IF
      IF LEFT$(str2, 6) = "LOCAL " THEN
         varDeclareSize = 6
      END IF
      IF LEFT$(str2, 7) = "STATIC " THEN
         varDeclareSize = 7
      END IF
      IF varDeclareSize = 0 THEN GOTO NoGlobalLocal
      varDeclare = LEFT$(str2, varDeclareSize)

      z = INSTR(str2, " AS ")
      IF z <= 0 THEN GOTO NoGlobalLocal ' old style?

      str3 = UCASE$(str1)

      y = INSTR(str2, ",")
      IF y <= 0 THEN ' ignore only 1 variable
         z = INSTR(str3, " AS ")
         IF z < preAS THEN
            b = preAS - z
            INCR b
            PRINT #fc.fhout, LEFT$(str1, z - 1); SPACE$(b); MID$(str1, z)
         ELSE
            str2 = LEFT$(str1, z - 1)
            str2 = RTRIM$(str2)
            a = LEN(str2)
            IF a < preAS THEN
               b = preAS - a
               PRINT #fc.fhout, LEFT$(str1, a); SPACE$(b); MID$(str1, z)
            ELSE
               PRINT #fc.fhout, str1
           END IF
         END IF
         ITERATE FOR
      END IF ' ignore only 1 variable

      str2 = UCASE$(str1)

      x = INSTR(str2, varDeclare)
      strBgn = LEFT$(str1, x + varDeclareSize - 1)
      z = INSTR(str2, " AS ")
      strAs = MID$(str1, z)

      x = x + varDeclareSize

      varstr = MID$(str1, x, z - x)
      y = preAS - LEN(strBgn)
      varfixsize = SPACE$(y)
      varcnt = PARSECOUNT(varstr, ",")
      RESET vars()
      PARSE varstr, vars(), ","
      FOR y = 0 TO varcnt - 1
         IF LEN(varfixsize) < LEN(TRIM$(vars(y))) THEN
            PRINT #fc.fhout, "*ERR "; varDeclare; STR$(LEN(varfixsize)); " but need "; STR$(LEN(TRIM$(vars(y))))
         END IF
         LSET varfixsize = TRIM$(vars(y))
         PRINT #fc.fhout, strBgn; varfixsize; strAs
      NEXT

      ITERATE FOR ' done with
      '  end of splitvariable code 06/21/2011 PDE

NoGlobalLocal:
      IF (currentline MOD 10) = 0 THEN           'update the progress control
         SendMessage fc.hProgress, %PBM_SETPOS, 100 * (CurrentLine / TotalLines), 0
      END IF
      WordCount = Line2Words(ztext, w()) ' , @fo)
      build = RebuildLine(w(), WordCount)
      IF LEN(w(0)) THEN                          'inline comment doesn't fit
         ' AND IS UGLY -- taken care of 11/18/2011
         PRINT #fc.fhout, w(0)                  'this increases the output file line count
         w(0) = ""
      END IF
      PRINT #fc.fhout, build
      ITERATE FOR
    NoGlobalLocal1:
      PRINT #fc.fhout, ztext
   NEXT
END SUB


FUNCTION Line2Words(BYREF work AS ASCIZ * %MAX_LINELEN, BYREF w() AS STRING ) AS LONG ' 11/18/2011
   '-------------------------------------------------------------------
   'Parse source line into word/token array w(). Handle special cases for
   'REM, DATA, string literals, ASM, !, '.
   '  work     [in/out] source code line text
   '  w()      [in/out] array of words/tokens
   ' NOPE  fo       [in/out] udt of formatting options
   'Returns count of words/tokens found
   '-------------------------------------------------------------------
   LOCAL stemp AS STRING
   LOCAL ncount AS LONG                          'word/token count
   LOCAL pchr AS BYTE PTR
   LOCAL s1 AS LONG

   ' REPLACE $TAB WITH SPACE$(fo.tabsize) IN work *** replaced with following 05/06/2011 PDE
   work = TAB$(work, fo.tabsize)

   work = S_TRIMBA(work)
   'X_AU "-----------------------------------"+$CRLF+TRIM$(work)+$CRLF+"--------------------------"
   stemp = UCASE$(TRIM$(work))                   'standardize incomming text
   '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   'These need no or limited processing
   '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   IF LEFT$(stemp, 1) = "'" OR _                 'remark
      LEN(stemp) = 0 OR _                           'blank line
      LEFT$(stemp, 5) = "DATA " THEN                'DATA statments ignored
         w(1) = TRIM$(work)
         m_ExitFunction(1)
   ELSEIF LEFT$(stemp, 4) = "REM " THEN
      w(1) = "'" + TRIM$(MID$(work, 5))          'replace REM with '
      m_ExitFunction(1)
   ELSEIF LEFT$(stemp, 4) = "ASM " THEN          'replace ASM with !
      w(1) = "!" + RTRIM$(EXTRACT$(MID$(work, 5), ANY "';"))
      w(2) = ";" + TRIM$(REMAIN$(work, ANY ";'"))
      m_ExitFunction(2)
   ELSEIF LEFT$(stemp, 1) = "!" THEN
      w(1) = TRIM$(EXTRACT$(work, ANY "';"))     'up to any comment
      w(2) = ";" + TRIM$(REMAIN$(work, ANY ";'"))
      m_ExitFunction(2)                          'two words/tokens
   END IF
   stemp = ""                                    'clear for reuse
   '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   'Parse remainder
   '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   pchr = VARPTR(work)                           'point to first byte
   DO
      IF @pchr = 34 THEN                         'quote start?
         GOSUB SaveWord                          'save any current word/token
         s1 = pchr                               'save string start
         DO
            INCR pchr                            'next string character
         LOOP UNTIL @pchr = 34                   'quoted string finished
         stemp = PEEK$(s1, pchr - s1 + 1)
         GOSUB SaveWord
         INCR pchr                               'next byte/character
         ITERATE DO
      END IF
      IF @pchr = 39 THEN                         'inline "'" remark
         GOSUB SaveWord
         'balance of line
         stemp = MID$(work, pchr - VARPTR(work) + 1)
         EXIT DO                                 'line done
      END IF
      '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      'Check for one of the standard delimters
      '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      IF INSTR( " ,\=+-/^*)(:[];><.", CHR$(@pchr)) THEN
         GOSUB SaveWord
         IF @pchr <> 32 THEN
            stemp = CHR$(@pchr)
            GOSUB SaveWord
         END IF
      ELSE
         stemp = stemp + CHR$(@pchr)             'add char to current token
      END IF
      INCR pchr
   LOOP WHILE @pchr <> 0                         'check for line end
   GOSUB SaveWord
   m_ExitFunction(ncount)                        'return token count

SaveWord:
   IF LEN(stemp) THEN                            'anything to do?
      INCR ncount                                'next array element
      w(ncount) = stemp                          'save it
      '   X_AU STR$(ncount)+"-"+stemp
      stemp = ""                                 'erase working string
   END IF
   RETURN

END FUNCTION

FUNCTION RebuildLine(BYREF w() AS STRING, BYREF wordcount AS LONG) AS STRING ' 11/18/2011
   '----------------------------------------------------------------
   'Reassemble a source line from the individual tokens in string
   'array Words() standardize puncuation, spacing and indentation.
   '  w()         [in/out] array of source words/tokens
   '  wordcount   [in/out] # words/tokens in array
   ' NOPE   fo          [in/out] udt of formatting options
   'Returns the formatted source code line
   '----------------------------------------------------------------
   LOCAL i& ' NASTY HABIT
   LOCAL stemp, comment, schr AS STRING
   LOCAL spacer AS LONG                          'helper with indenting
   LOCAL WordNo AS LONG

   schr = LEFT$(w(1), 1)
   IF schr = "'" THEN
      m_ExitFunction(SPACE$(fc.indent) + w(1))   'return full comment line
   ELSEIF UCASE$(LEFT$(w(1), 5)) = "DATA" THEN   'line is data
      m_ExitFunction(w(1))                       'return unmodified
   ELSEIF schr = "!" THEN
      IF LEN(w(2)) > 1 THEN                      'inline assembler comment
         stemp = LSET$(SPACE$(fc.indent) + w(1), fo.remcol - 1) + w(2)
      ELSE
         stemp = SPACE$(fc.indent) + w(1)
      END IF
      m_ExitFunction(stemp)
   END IF
   '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   'If last item in token array is a inline comment, save it to
   'be inserted later.
   '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   schr = LEFT$(w(WordCount), 1)
   IF schr = "'" OR schr = ";" THEN
      comment = w(WordCount)
      IF LEN(comment) <= 1 THEN
         comment = ""                            'make blank line
      END IF
      DECR WordCount
   END IF
   IF fo.lineupequates THEN
      schr = LEFT$(w(1), 1)
      IF (schr = "%" OR schr = "$") AND w(2) = "=" THEN
         IF fo.capequates THEN
            w(1) = UCASE$(w(1))
         END IF
         i& = MAX&(LEN(w(1)) + 2, fo.equatescolumn)
         stemp = LSET$(w(1), i& - 2) + " = "
         FOR i& = 3 TO wordcount
            stemp = stemp + TRIM$(w(i&))
         NEXT
         IF LEN(comment) AND LEN(stemp) < fo.remcol - 1 THEN
            stemp = LSET$(stemp, fo.remcol - 1) + TRIM$(comment)
         ELSE
            'w(0) = comment
            stemp = RTRIM$(stemp) & SPACE$(fo.tabsize) & comment ' 11/18/2011 PDE
            comment = ""
         END IF
         m_ExitFunction(stemp)
      END IF
   END IF
   '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   'process each token in array w()
   '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   stemp = ""
   DO
      INCR WordNo                                'first/next word
      '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      'Is word abreviated print statement ?. Expand and capitilize
      '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      schr = LEFT$(w(WordNo), 1)
      IF w(WordNo) = "?" THEN                    'expand to PRINT (PBCC)
         w(WordNo) = "PRINT"
      ELSEIF schr = $DQ THEN                     'string literal
         stemp = stemp + w(WordNo) + " "         'simply add it
         ITERATE DO
      ELSEIF schr = "%" OR schr = "$" THEN       'equates
         IF fo.capequates THEN
            w(WordNo) = UCASE$(w(WordNo))
         END IF
      ELSEIF schr = "!" THEN
         stemp = stemp + w(WordNo)               'assembler line
         spacer = fc.indent
         fc.indent = 0
         EXIT DO
      END IF

      SELECT CASE UCASE$(w(WordNo))              'process the token
            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            'Check to see if token is a label (wordcount = 2), only thing
            'allowed on a line with a label is a comment and that was
            'removed above. Should it fail this test its assumed to be
            'part of a multi-statement line. Labels always start in col 1
            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         CASE ":"                                'check for label
            stemp = RTRIM$(stemp)
            IF WordCount = 2 THEN                'we assume a label (name + :)
               spacer = fc.indent                'save indent value
               fc.indent = 0                     'move label to left margin
            END IF
            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            'Try to distinguish between array and something else as to
            'whether or not there should be a space preecding these. Need
            'to test for a PB intrinsic function like ATTRIB(xxx) vs OR
            '(xx + yy).
            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         CASE "(", "["
            IF INSTR( "+-*/\=><", w(WordNo - 1)) = 0 THEN
               stemp = RTRIM$(stemp)
            END IF
            SELECT CASE UCASE$(w(WordNo - 1))
               CASE "IF", "ELSEIF", "AND", "OR", "NOT", "ISFALSE", "ISTRUE", "XOR", "TO"
                  stemp = stemp + " "            'add spacing
            END SELECT
            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            'We never want any space to proceed these delimiters
            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         CASE ")", ",", "]", "[", ";", "."
            stemp = RTRIM$(stemp)                'no preceding space
            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            'Catch <>, >=, <= combinations to eliminate < >, > =,. etc.
            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         CASE "<", ">", "="
            IF INSTR( "<>=+-*ORAND", w(WordNo - 1)) THEN ' 05/06/2011 PDE added + - * not sure how to handle OR AND
               stemp = RTRIM$(stemp)
            END IF
            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            'Compare last token to THEN, nothing else on line means started
            'a multi-line IF block. Any comment already removed above.
            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         CASE "THEN"
            IF WordNo = WordCount THEN
               spacer = spacer + fo.tabsize      'change for next line
            END IF
         CASE "TYPE", "UNION", "TRY", "FUNCTION", "SUB", "INTERFACE", "PROPERTY"
            IF (WordNo = 1) AND (w(WordNo + 1) <> "=") THEN spacer = spacer + fo.tabsize

         CASE "CLASS"
            IF WordNo = 1 AND UCASE$(w(WordNo + 1)) <> "METHOD" THEN
               INCR fc.inClass
            END IF
            IF WordNo = 1 AND w(WordNo + 1) <> "=" THEN spacer = spacer + fo.tabsize

         CASE "METHOD"
            IF WordNo = 1 AND fc.inClass > 0 THEN
               spacer = spacer + fo.tabsize ' next line
            END IF

            '         case "FOR"
            '          if Is_there(w(),"NEXT",WordCount) then
            '             IF (WordNo = 1) THEN spacer = spacer + fo.tabsize
            '          end if
            Construct( "FOR", "NEXT")
            Construct( "WHILE", "WEND")
            Construct( "DO", "LOOP")

         CASE "MACRO"
            IF WordNo = 1 AND UCASE$(w(2)) = "FUNCTION" THEN
               spacer = spacer + fo.tabsize
            END IF
         CASE "CALLBACK" ' 05/09/2011 PDE need to indent same as normal FUNCTION
            IF WordNo = 1 AND UCASE$(w(2)) = "FUNCTION" THEN
               spacer = spacer + fo.tabsize
            END IF
         CASE "SELECT"
            IF WordNo = 1 THEN spacer = spacer + (fo.tabsize * 2)
            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            'Handle ELSEIF seperately since it appears on same line with THEN
            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         CASE "#IF"                              'these require no line ending THEN
            spacer = spacer + fo.tabsize
         CASE "ELSEIF", "#ELSEIF", "#ELSE"
            fc.indent = fc.indent - fo.tabsize   'change for this line
         CASE "#ENDIF"
            fc.indent = fc.indent - fo.tabsize
            spacer = 0
         CASE "ELSE", "CASE"
            IF WordNo = 1 THEN                   'first word?
               spacer = spacer + fo.tabsize      'change for next line
               fc.indent = fc.indent - fo.tabsize'change for this line
            END IF
         CASE "END"
            SELECT CASE UCASE$(w(WordNo + 1))
               CASE "TYPE", "IF", "UNION", "SUB", "FUNCTION", "INTERFACE", "TRY", "MACRO", "METHOD", "PROPERTY"
                  fc.indent = fc.indent - fo.tabsize
               CASE "CLASS"
                  DECR fc.inClass
                  spacer = 0 ' next line
                  fc.indent = fc.indent - fo.tabsize ' this line
               CASE "SELECT"
                  fc.indent = fc.indent - (fo.tabsize * 2)
            END SELECT
            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            'Check for the close of a looping block. If its on the same
            'line as loop start, keep same indent.
            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         CASE "WEND", "NEXT", "LOOP"
            IF wordno = 1 THEN
               schr = UCASE$(stemp)              'standardize
               spacer = 0                        'be sure this = 0
               fc.indent = fc.indent - fo.tabsize'default is close loop
               IF INSTR(schr, "DO ") THEN
                  fc.indent = fc.indent + fo.tabsize
               ELSEIF INSTR(schr, "WHILE ") THEN
                  fc.indent = fc.indent + fo.tabsize
               ELSEIF INSTR(schr, "ITERATE") THEN
                  fc.indent = fc.indent + fo.tabsize
               END IF
            END IF
      END SELECT
      '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      'No space following "([]"
      '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      IF w(wordno) = "." THEN
         stemp = RTRIM$(stemp) + "."
      ELSEIF INSTR( "(][", w(WordNo - 1)) THEN
         stemp = RTRIM$(stemp) + w(WordNo) + " "
      ELSE
         stemp = stemp + w(WordNo) + " "
      END IF
   LOOP WHILE WordNo < WordCount
   IF fc.indent < 0 THEN fc.indent = 0           'stay at left margin
   stemp = SPACE$(fc.indent) + RTRIM$(stemp)     'final formated line
   '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   'Add any line comment preserved above. Fit failure returns
   'inline comment in w(0)
   '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   w(0) = ""
   IF LEN(comment) THEN
      IF LEN(stemp) < fo.remcol THEN
         stemp = LEFT$(stemp + SPACE$(fo.RemCol - 1), fo.RemCol - 1) + comment
      ELSE
         ' w(0) = SPACE$(fo.indent) + comment
         stemp = RTRIM$(stemp) & SPACE$(fo.tabsize) & comment ' 11/18/2011
         comment = ""
      END IF
   END IF
   fc.indent = fc.indent + spacer                'new indent value
   FUNCTION = stemp                              'return formatted source line
END FUNCTION

'----------------------------------------------------------------

FUNCTION Is_there(BYREF w() AS STRING, BYVAL S01 AS STRING, BYVAL wc AS LONG) AS LONG
   REGISTER R01 AS LONG, R02 AS LONG
   R02 = 0
   FOR R01 = 2 TO wc
      IF TRIM$(UCASE$(w(R01))) = UCASE$(S01) THEN R02 = 1
   NEXT
   FUNCTION = 1 - R02                            ' Inverted for easier usage
END FUNCTION

'// Get window texte.
FUNCTION zGetCTLText(BYVAL hCtrl AS LONG) AS XSTRING
    LOCAL szText AS XASCIIZ * 4096, Length AS LONG
    Length = GetWindowText(hCtrl, szText, SIZEOF(szText))
    IF Length THEN FUNCTION = LEFT$(szText, Length)
END FUNCTION

'// Changes control text.
SUB zSetCTLText(BYVAL hCTRL AS LONG, zText AS XASCIIZ)
    IF hCtrl THEN
       IF zText <> zGetCTLText(hCtrl) THEN CALL SendMessage(hCTRL, %WM_SETTEXT, 0, VARPTR(zText))
    END IF
END SUB

SUB zSetCheckButtonStatus(BYVAL hBut AS LONG, BYVAL TrueFalse AS LONG)
    LOCAL UseState AS LONG
    IF TrueFalse THEN UseState = %BST_CHECKED ELSE UseState = %BST_UNCHECKED
    CALL SendMessage(hBut, %BM_SETCHECK, UseState, 0)
END SUB

FUNCTION zGetCheckButtonStatus(BYVAL hBut AS LONG) AS LONG
    IF SendMessage(hBut, %BM_GETCHECK, 0, 0) = %BST_CHECKED THEN
       FUNCTION = -1
    END IF
END FUNCTION

SUB GetOVars()
  IF  zGetCheckButtonStatus(GetDlgItem (tabhandle(1), %IDC_FORM1_CHECK_CREATE_BAK)) THEN
     fo.backup = 1
  ELSE
     fo.backup = 0
  END IF

  IF zGetCheckButtonStatus(GetDlgItem (tabhandle(1), %IDC_FORM1_CHECK_CREATE_TIME_STAMP)) THEN
    fo.backupTimeStamp = 1
  ELSE
    fo.backupTimeStamp = 0
  END IF

  IF zGetCheckButtonStatus(GetDlgItem (tabhandle(1), %IDC_FORM1_CHECK_SPLIT_VARIABLES)) THEN
    fo.splitvariables = 1
  ELSE
    fo.splitvariables = 0
  END IF

  IF zGetCheckButtonStatus(GetDlgItem (tabhandle(1), %IDC_FORM1_CHECK_UPCASE_EQUATES)) THEN

    fo.capequates = 1
  ELSE
    fo.capequates = 0
  END IF


  IF zGetCheckButtonStatus(GetDlgItem (tabhandle(1), %IDC_FORM1_CHECK_FORMAT_EQULATES)) THEN

     fo.lineupequates = 1
  ELSE
     fo.lineupequates = 0
  END IF

  fo.equatescolumn = VAL(zGetCTLText(GetDlgItem(tabhandle(1), %IDC_FORM1_TEXT_EQUATECOLUMN)))
  fo.tabsize = VAL(zGetCTLText(GetDlgItem(tabhandle(1), %IDC_FORM1_TEXT_TABSIZE)))
  fo.remcol = VAL(zGetCTLText(GetDlgItem(tabhandle(1), %IDC_FORM1_TEXT_REMCOLUMN)))

END SUB

SUB PutOVar

  IF fo.backup THEN
    zSetCheckButtonStatus(GetDlgItem (tabhandle(1), %IDC_FORM1_CHECK_CREATE_BAK), %TRUE)

  ELSE
    zSetCheckButtonStatus(GetDlgItem (tabhandle(1), %IDC_FORM1_CHECK_CREATE_BAK), %FALSE)
    Control_Disable GetDlgItem (tabhandle(1), %IDC_FORM1_CHECK_CREATE_TIME_STAMP)
  END IF

  IF fo.backupTimeStamp THEN
    zSetCheckButtonStatus(GetDlgItem (tabhandle(1), %IDC_FORM1_CHECK_CREATE_TIME_STAMP), %TRUE)
  ELSE
    zSetCheckButtonStatus(GetDlgItem (tabhandle(1), %IDC_FORM1_CHECK_CREATE_TIME_STAMP), %FALSE)
  END IF
  IF fo.splitvariables THEN
    zSetCheckButtonStatus(GetDlgItem (tabhandle(1), %IDC_FORM1_CHECK_SPLIT_VARIABLES), %TRUE)
  ELSE
    zSetCheckButtonStatus(GetDlgItem (tabhandle(1), %IDC_FORM1_CHECK_SPLIT_VARIABLES), %FALSE)
  END IF

  IF fo.capequates THEN
    zSetCheckButtonStatus(GetDlgItem (tabhandle(1), %IDC_FORM1_CHECK_UPCASE_EQUATES), %TRUE)
  ELSE
    zSetCheckButtonStatus(GetDlgItem (tabhandle(1), %IDC_FORM1_CHECK_UPCASE_EQUATES), %FALSE)
  END IF

  IF fo.lineupequates THEN
     zSetCheckButtonStatus(GetDlgItem (tabhandle(1), %IDC_FORM1_CHECK_FORMAT_EQULATES), %TRUE)
  ELSE
     zSetCheckButtonStatus(GetDlgItem (tabhandle(1), %IDC_FORM1_CHECK_FORMAT_EQULATES), %FALSE)
     Control_Disable GetDlgItem (tabhandle(1), %IDC_FORM1_CHECK_UPCASE_EQUATES)
     Control_Disable GetDlgItem (tabhandle(1), %IDC_FORM1_TEXT_EQUATECOLUMN)
  END IF

  zSetCTLText(GetDlgItem(tabhandle(1), %IDC_FORM1_TEXT_EQUATECOLUMN), FORMAT$(fo.equatescolumn))
  zSetCTLText(GetDlgItem(tabhandle(1), %IDC_FORM1_TEXT_TABSIZE), FORMAT$(fo.tabsize))
  zSetCTLText(GetDlgItem(tabhandle(1), %IDC_FORM1_TEXT_REMCOLUMN), FORMAT$(fo.remcol))

END SUB

SUB Control_Disable( BYVAL hWndControl AS XHWND )

    ' Do a check to ensure that this is actually a window handle
    IF IsWindow(hWndControl) THEN

       ' Disable the window.
         EnableWindow hWndControl, %FALSE

    END IF

END SUB

SUB Control_Enable(BYVAL hWndControl AS XHWND)
    IF IsWindow(hWndControl) THEN

       ' Disable the window.
         EnableWindow hWndControl, %TRUE

    END IF
END SUB

SUB CopyBak()
  LOCAL p, f, fout, d, t AS STRING

  f = PATHNAME$(NAME, fo.fspecin)
  IF fo.backup THEN
    IF fo.backupTimeStamp = 0 THEN
       p = PATHNAME$(PATH, fo.fspecin)
       fout = p + f + ".BAK"

        CopyFile fo.fspecin, (fout), 0
       ELSE
         p = PATHNAME$(PATH, fo.fspecin) + "BAK"
         IF ISFOLDER(p) = 0 THEN
            MKDIR p
            p = p + "\"
         END IF
         d = Built.DateString
         REPLACE "." WITH "_" IN d
         t = Built.TimeString
         REPLACE ":" WITH "_" IN t

         CopyFile fo.fspecin, p + "\" + f + "_"+ d + " " + t + " .BAK", 0


       END IF
   END IF
END SUB

